import json
from pathlib import Path
import epdx

import requests

OKOBAU_URL = "https://oekobaudat.de/OEKOBAU.DAT/resource/datastocks/cd2bda71-760b-4fcc-8a0b-3877c10000a8"


def get_folder(source, name: str) -> Path:
    """Get or create a folder with the given name"""

    folder = Path(source).parent / name
    if not folder.exists():
        folder.mkdir()

    return folder


def get_compliance_id(value: str) -> str:
    compliance_values = {
        "a1": "b00f9ec0-7874-11e3-981f-0800200c9a66",
        "a2": "c0016b33-8cf7-415c-ac6e-deba0d21440d",
    }

    return compliance_values.get(value.lower())


def get_epds(search: dict[str, str] | None, limit=10) -> dict:
    """Get EPDs from Ökobau"""

    response = requests.get(
        f"{OKOBAU_URL}/processes?"
        f"format=json"
        f"&pageSize={limit}"
        f"{'&search=true&' + '&'.join([f'{key}={value}' for key, value in search.items()]) if search else ''}"
    )
    response.raise_for_status()
    data = response.json()

    print(
        f"Retrieved {data.get('pageSize')} EPDs out of {data.get('totalCount')} from Ökobau"
    )

    return data


def get_full_epd(uid: str) -> dict:
    """Get the full dataset for a single EPD"""

    base_url = f"{OKOBAU_URL}/processes/{uid}"
    response = requests.get(f"{base_url}?format=json&view=extended")

    response.raise_for_status()
    data = response.json()
    data["source"] = base_url

    return data


def download_to_epdx(data_folder: Path, search_key: str, limit: int = 50):
    """Download 50 EPDs and save them locally as EPDx files"""

    data = get_epds({"name": search_key, "compliance": get_compliance_id("a1")}, limit=limit)
    print(f"Downloading {limit} EPDs\n")

    for _epd in data.get("data"):
        epd_id = _epd.get("uuid")

        epdx_file_path = data_folder / f"{epd_id}.epdx.json"
        if not epdx_file_path.exists():
            print(f"Downloading EPD {epd_id[:8]} to {'/'.join(data_folder.parts[-3:])}")

            # Get full EPD from Ökobau
            epd = get_full_epd(epd_id)

            # Turn EPD into EPDx JSON string that can be saved to disk.
            try:
                epdx_data = epdx.convert_ilcd(epd, as_type=str)
                epdx_file_path.write_text(epdx_data)
            except epdx.ParsingException:
                print(f"Error ID: {epd_id}")
                (data_folder / f"{epd_id}.json").write_text(json.dumps(epd))
        else:
            print(f"EPD {epd_id[:8]} already in {'/'.join(data_folder.parts[-3:])}")
